@@ -25,18 +25,13 @@
 
 #include "config.h"
 
-#ifdef HAVE_LIBVIRT
-#include <libvirt/libvirt.h>
-#endif
-
 #include "virt-viewer-session.h"
-#include "virt-viewer-util.h"
 
-int virt_viewer_auth_collect_credentials(GtkWindow *window,
-                                         const char *type,
-                                         const char *address,
-                                         char **username,
-                                         char **password);
+gboolean virt_viewer_auth_collect_credentials(GtkWindow *window,
+                                              const char *type,
+                                              const char *address,
+                                              char **username,
+                                              char **password);
 
 #endif
 /*
