@@ -56,12 +56,6 @@ virt_viewer_notebook_set_property (GObject *object, guint property_id,
     }
 }
 
-static void
-virt_viewer_notebook_dispose (GObject *object)
-{
-    G_OBJECT_CLASS (virt_viewer_notebook_parent_class)->dispose (object);
-}
-
 static void
 virt_viewer_notebook_class_init (VirtViewerNotebookClass *klass)
 {
@@ -71,14 +65,12 @@ virt_viewer_notebook_class_init (VirtViewerNotebookClass *klass)
 
     object_class->get_property = virt_viewer_notebook_get_property;
     object_class->set_property = virt_viewer_notebook_set_property;
-    object_class->dispose = virt_viewer_notebook_dispose;
 }
 
 static void
 virt_viewer_notebook_init (VirtViewerNotebook *self)
 {
     VirtViewerNotebookPrivate *priv;
-    GdkColor color;
 
     self->priv = GET_PRIVATE(self);
     priv = self->priv;
@@ -88,8 +80,6 @@ virt_viewer_notebook_init (VirtViewerNotebook *self)
     gtk_notebook_set_show_border(GTK_NOTEBOOK(self), FALSE);
     gtk_widget_show_all(priv->status);
     gtk_notebook_append_page(GTK_NOTEBOOK(self), priv->status, NULL);
-    gdk_color_parse("white", &color);
-    gtk_widget_modify_fg(priv->status, GTK_STATE_NORMAL, &color);
 }
 
 void
