@@ -30,10 +30,11 @@ extern gboolean doDebug;
 
 enum {
     VIRT_VIEWER_ERROR_FAILED,
-    VIRT_VIEWER_ERROR_CANCELLED,
+    VIRT_VIEWER_ERROR_CANCELLED
 };
 
 #define VIRT_VIEWER_ERROR virt_viewer_error_quark ()
+#define VIRT_VIEWER_RESOURCE_PREFIX "/org/virt-manager/virt-viewer"
 
 GQuark virt_viewer_error_quark(void);
 
@@ -60,6 +61,10 @@ gint virt_viewer_compare_buildid(const gchar *s1, const gchar *s2);
 void virt_viewer_align_monitors_linear(GHashTable *displays);
 void virt_viewer_shift_monitors_to_origin(GHashTable *displays);
 
+/* monitor mapping */
+GHashTable* virt_viewer_parse_monitor_mappings(gchar **mappings,
+                                               const gsize nmappings,
+                                               const gint nmonitors);
 #endif
 
 /*
